@@ -19,6 +19,7 @@ import records #record data
 
 #own functions
 from functions_general import *
+from functions_record import *
 
 try:
     import alarm #maemo specific
@@ -32,7 +33,7 @@ _ = i18n.language.ugettext
 
 
 ###########################################################################
-##
+## Sets the MAEMO alarm
 ###########################################################################
 def SetAlarm():
     if(data.demo==False): #We are on a N900
@@ -65,7 +66,7 @@ def SetAlarm():
 
 
 ###########################################################################
-## Delete al alarms with appid "SleepAnalyser"
+## Delete all MAEMO alarms with appid "SleepAnalyser"
 ## When SleepAnalyser crashed, it is possible that there is still an (old) alarm  in the dbus alarm queue
 ## This function deletes them all
 ###########################################################################
@@ -92,7 +93,7 @@ def delete_all_preexisting_Alarms():
 
 
 ###########################################################################
-##
+## Deletes the set MAEMO alarm
 ###########################################################################
 def DeleteAlarm():
     if(data.AlarmID!=-1): #there is an old alarm to be deleted
@@ -115,7 +116,6 @@ def DeleteAlarm():
     else:
         if(data.debug==True): print "* No alarm to be deleted"
 
-#    records.AlarmIsSet=False
     data.AlarmIsActive=False
 
 
@@ -123,8 +123,10 @@ def DeleteAlarm():
 
 
 
+
 ###########################################################################
-##
+## Gets called in the timer loop
+## Checks if the alarm should be set off
 ###########################################################################
 def ProcessAlarm():
     i=records.Index-data.Derivation
@@ -134,29 +136,52 @@ def ProcessAlarm():
     now=Tuple2Seconds(time.localtime())
 #    if(peak>data.Trigger): #big Movement
     if(peak>(data.Trigger*data.Yscale_Sleep_Pattern*3/40)): #Trigger Alarm
-       if(records.AlarmIsSet==True and data.AlarmIsActive==False and now>=records.AlarmTimeWindowStart_seconds and now<records.AlarmTimeWindowEnd_seconds):
-            #print big movement and we are in alarm time window, update alarm time, set alarm off
-            print "************ ALARM ALARM (Movement over trigger level)"
-            print "*", strftime("%x %X"), "ProcessAlarm => DeleteAlarm" #for log file
-            DeleteAlarm() #delete old alarm if existing
-#            h=int(strftime("%H", time.localtime()))
-#            m=int(strftime("%M", time.localtime()))
-#            print "* Set new Alarm to current time:",  h,  ":",  m
-#            now=int(strftime("%H", time.localtime()))*3600+int(strftime("%M", time.localtime()))*60
-            records.AlarmTime_tuple=time.localtime()#set to now
-            records.AlarmTime_seconds=Tuple2Seconds(records.AlarmTime_tuple)
-            print "*", strftime("%x %X"), "ProcessAlarm => SetAlarm" #for log file
-            SetAlarm() #set alarm to go off now
-            records.Trigger=data.Trigger #update Trigger level on alarm occurence
-            data.AlarmIsActive=True
+       if(records.AlarmIsSet==True and data.AlarmIsActive==False): #alarm is set
+            if(now>=records.AlarmTimeWindowStart_seconds and now<records.AlarmTimeWindowEnd_seconds): # we are in alarm time window
+                if(data.WakeUp_SongIsActive==False):#wake up song (if enabled) is not playing
+                    print "************ ALARM ALARM (Movement over trigger level)"
+                    print "*", strftime("%x %X"), "ProcessAlarm => DeleteAlarm" #for log file
+                    DeleteAlarm() #delete old alarm if existing
+                    records.AlarmTime_tuple=time.localtime()#set to now
+                    records.AlarmTime_seconds=Tuple2Seconds(records.AlarmTime_tuple)
+
+                    if(data.Use_WakeUp_Song==True): #Delay alarm, so we can first play wake up song
+                        print "* Alarm would be now:", records.AlarmTime_tuple
+                        print "* But wake Up song is enabled, delay alarm for max", data.WakeUp_Song_Duration/60, "minutes"
+                        records.AlarmTime_seconds=min(records.AlarmTime_seconds+data.WakeUp_Song_Duration, records.AlarmTimeWindowEnd_seconds) #add delay time, but max until alarm time window
+                        records.AlarmTime_tuple=Seconds2Tuple(records.AlarmTime_seconds)
+                        data.WakeUp_SongIsActive=True
+                        records.WakeUp_Song_Start_tuple=Seconds2Tuple(now)
+                        print "* Delay alarm until:", records.AlarmTime_tuple
+
+                        print "** Play wake up song"
+                        PlaySong(data.WakeUp_Song_Volume, data.WakeUp_Song)
+
+                    print "*", strftime("%x %X"), "ProcessAlarm => SetAlarm" #for log file
+                    SetAlarm() #set alarm to go off now (or with wake up song delay)
+
+                if(now >= records.AlarmTime_seconds): #alarm is due
+                    records.Trigger=data.Trigger #update Trigger level on alarm occurence
+                    data.AlarmIsActive=True
+
     elif(now>=records.AlarmTimeWindowEnd_seconds): #End of alarm time window, set alarm off anyway
         if(records.AlarmIsSet==True and data.AlarmIsActive==False): #Alarm is set and not active yet
             print "************ ALARM ALARM (End of Alarm Time Window)"
-#            records.AlarmTime_tuple=time.localtime()#set to now (just for record)
-#            records.AlarmTime_seconds=Tuple2Seconds(records.AlarmTime_tuple)
-#            records.Trigger=data.Trigger #update Trigger level on alarm occurence
+            records.Trigger=data.Trigger #update Trigger level on alarm occurence
             data.AlarmIsActive=True
 
-#    print "records.AlarmIsSet", records.AlarmIsSet
-#    print "data.AlarmIsActive:",  data.AlarmIsActive
-#    print ""
+
+
+
+###########################################################################
+## Checks if it is time to start the ucid dream song
+###########################################################################
+def ProcessLucid_Dream(): #Lucid dream option
+    if(data.Use_Lucid_Dream_Song==True and data.Lucid_Dream_SongIsActive==False):
+        GenerateStatisticData()
+#        print records.arrStatData[1][2], records.arrStatData[2][2]
+        if(records.arrStatData[1][2] >= data.Lucid_Dream_Song_Delay or records.arrStatData[2][2] >= data.Lucid_Dream_Song_Delay):
+            data.Lucid_Dream_SongIsActive=True
+            records.Lucid_Dream_Song_Start_tuple=time.localtime() #now
+            print "Play Lucid dream song"
+            PlaySong(data.Lucid_Dream_Song_Volume, data.Lucid_Dream_Song)
